import numpy as np
import math
import lib_gcode
import lib_drawing

x_max = 205
y_max = 128
z_up = 2.0
z_down = -1.5


def main_repeat():
    n_shapes = 30

    square = np.array([[1, 1],
                       [-1, 1],
                       [-1, -1],
                       [1, -1],
                       [1, 1]])
    shape = lib_drawing.repeat_shape(square, 0.05, 0.95, n_shapes)
    shape = lib_drawing.fit_rectangle(shape, 0, 0, x_max, y_max)

    writer = lib_gcode.GcodeWriter(z_up=z_up, z_down=z_down)
    
    for i in range(n_shapes):
        writer.add_curve(shape[i, :, :], lift=True)
    writer.goto(0, 120)

    lib_drawing.plot_curves(shape)

    shoot_gcode = lib_gcode.open_gcode("shoot.gcode")
    writer.gcode = lib_gcode.insert_gcode(writer.gcode, shoot_gcode, t=5)
    writer.save("test.gcode")

    
def main_spiro():
    r1 = 100
    r2 = 2
    r3 = 80
    
    shape = lib_drawing.spirograph(r1, r2, r3)
    shape = lib_drawing.fit_rectangle(shape, 0, 0, x_max, y_max)
    
    writer = lib_gcode.GcodeWriter(z_up=z_up, z_down=z_down)
    writer.add_curve(shape)
    writer.goto(0, 120)
    
    lib_drawing.plot_curves(shape)

    shoot_gcode = lib_gcode.open_gcode("shoot.gcode")
    writer.gcode = lib_gcode.insert_gcode(writer.gcode, shoot_gcode, t=5)
    writer.save("test.gcode")


def main_sinc():
    n = 8
    n_x = 127

    x = np.linspace(0, 1.3*math.pi, n_x)
    y = np.sinc(x)

    amp = np.linspace(2, 0, n)

    shape = np.zeros((n, n_x, 2))

    for i in range(n):
        shape[i, :, 0] = x
        shape[i, :, 1] = amp[i]*y

    shape = lib_drawing.fit_rectangle(shape, 0, 0, x_max, y_max)

    lib_drawing.plot_curves(shape)

    writer = lib_gcode.GcodeWriter(z_up=z_up, z_down=z_down)
    for i in range(len(shape)):
        writer.add_curve(shape[i, :, :], lift=True)
    writer.goto(0, 120)
    writer.save("test.gcode")


def main_repeat_spiro():
    r1 = 12
    r2 = 8
    r3 = 5

    n = 16

    shape = lib_drawing.spirograph(r1, r2, r3)

    shape = lib_drawing.repeat_shape(shape, 0.2, 0.9, n)

    shape = lib_drawing.fit_rectangle(shape, 0, 0, x_max, y_max)

    lib_drawing.plot_curves(shape)

    writer = lib_gcode.GcodeWriter(z_up=z_up, z_down=z_down)
    for i in range(len(shape)):
        writer.add_curve(shape[i, :, :], lift=True)
    writer.goto(0, 120)
    writer.save("test.gcode")


if __name__ == "__main__":
    main_spiro()
